#include "WorldClock.h"
#include <unistd.h>
#include "dtime.h"

#ifdef WIN32
#include "windows.h"
#include "winbase.h"
#include "winnt.h"
#endif

WorldClock::WorldClock()
{
	now = 0;
	mult = 1.0;
	paused = true;
	paused_m = SDL_CreateMutex();
	paused_c = SDL_CreateCond();;
}

WorldClock::~WorldClock()
{
}

void WorldClock::AdvanceTime(double amount)
{
	now += amount;
}

double WorldClock::BeginTimedCycle()
{
	return dtime();
}

bool WorldClock::DelayTime(double len, double tcl)
{
	SDL_mutexP(paused_m);
	if(paused) {
		SDL_CondWait(paused_c, paused_m);
		SDL_mutexV(paused_m);
		return true;
	} else {
		SDL_mutexV(paused_m);
		double n = dtime();
		double waitlen = len - ((n - tcl) * mult);
		if(waitlen < 0) return false;
#ifndef WIN32
		usleep((long)(1000000.0 * waitlen / mult));
#else
        ::Sleep((int)(waitlen * 1000.0 / mult));
#endif
		return true;
	}
}

void WorldClock::SetTimeMultiplier(double m)
{
	mult = m;
}

double WorldClock::GetTimeMultiplier()
{
	return mult;
}

void WorldClock::SetPaused(bool p)
{
	SDL_mutexP(paused_m);
	if(paused == true && p == false) {
		paused = false;
		SDL_CondBroadcast(paused_c);
	} else {
		paused = p;
	}
	SDL_mutexV(paused_m);
}

bool WorldClock::IsPaused()
{
	return paused;
}

