#ifndef __user_code_h__
#define __user_code_h__

#include "RobotIF.h"
#include "ObjectTypes.h"
#include "DifferentialDriveIntegrator.h"
#include <string>
#include <vector>

using namespace std;

class RobotIF;

enum {
	MODE_MANUAL,
	MODE_AUTOMATIC,
	MODE_WP,
};

class UserCode {
public:
	UserCode(RobotIF &robot);
	~UserCode();

	void setup();
	void handle_param(string key, string val);

	void cycle();

	void key_press(char k);
	void mouse_click(int x, int y, double worldx, double worldy, int button);

	void collision(int object_type);

	void get_log_data(double[]);
	int num_log_vars();

private:
	RobotIF &robot;
	DifferentialDriveIntegrator integrator;

	double last_call;
	long last_lticks;
	long last_rticks;

	int mode;

	double fwspeed, rspeed;

	void toggle_mode();

	/* internal wp stuff */
	vector<pair<double, double> > waypoints;
	vector<pair<double, double> >::iterator wp_it;
	bool auto_wp;
};

#endif
